heist_outfit_selector.Options:PostInit()

local function is_modded_outfit_variation(outfit, variation)
    if tweak_data.blackmarket.player_styles[outfit] ~= nil
            and tweak_data.blackmarket.player_styles[outfit].material_variations ~= nil
            and tweak_data.blackmarket.player_styles[outfit].material_variations[variation] ~= nil then
        return tweak_data.blackmarket.player_styles[outfit].material_variations[variation].custom == true
    else
        return false
    end
end

-- Replicated/extracted from BlackMarketGui:populate_gloves()
local function is_glove_locked(glove)
    local is_infamy_locked = managers.dlc:is_content_infamy_locked("gloves", glove)

    local is_dlc_locked = tweak_data.blackmarket.gloves[glove].global_value ~= nil
            and not managers.dlc:is_global_value_unlocked(tweak_data.blackmarket.gloves[glove].global_value)

    local glove_achievement_lock_id = managers.dlc:achievement_locked_content().gloves[glove]
    local dlc_tweak = glove_achievement_lock_id and tweak_data.dlc[glove_achievement_lock_id]
    local achievement_info = managers.achievment:get_info(dlc_tweak and dlc_tweak.achievement_id)
    local is_achievement_locked = achievement_info and not achievement_info.awarded

    return is_infamy_locked or is_dlc_locked or is_achievement_locked
end

-- Replicated/extracted from BlackMarketGui:populate_suit_variations()
local function is_outfit_or_variation_locked(outfit, variation)
    local is_infamy_locked = managers.dlc:is_content_infamy_locked("player_styles", outfit)

    local outfit_data = tweak_data.blackmarket.player_styles[outfit]
    local variation_data = outfit_data and tweak_data.blackmarket.player_styles[outfit].material_variations
            and tweak_data.blackmarket.player_styles[outfit].material_variations[variation]
    local variation_global_value = variation_data and variation_data.global_value or "normal"

    local is_dlc_locked = tweak_data.lootdrop.global_values[variation_global_value]
            and tweak_data.lootdrop.global_values[variation_global_value].dlc
            and not managers.dlc:is_dlc_unlocked(variation_global_value)

    local achievement = variation_data and variation_data.locks and variation_data.locks.achievement
    if variation == "default" then
        achievement = achievement or outfit_data and outfit_data.locks and outfit_data.locks.achievement
    end
    local is_achievement_locked = achievement and managers.achievment:get_info(achievement)
            and not managers.achievment:get_info(achievement).awarded

    local is_otherwise_locked = not managers.blackmarket:suit_variation_unlocked(outfit, variation)

    return is_infamy_locked or is_dlc_locked or is_achievement_locked or is_otherwise_locked
end

local function get_taken_outfit_variations(outfit)
    local should_include_modded_outfit_variations = heist_outfit_selector:should_include_modded_outfit_variations()
    local taken_variations = {}
    for _, character in ipairs(managers.criminals._characters) do
        if character ~= nil
                and character.taken == true
                and character.visual_state ~= nil
                and character.visual_state.player_style == outfit
                and character.visual_state.suit_variation ~= nil
                and not table.contains(taken_variations, character.visual_state.suit_variation)
                and (not is_modded_outfit_variation(outfit, character.visual_state.suit_variation) or should_include_modded_outfit_variations) then
            table.insert(taken_variations, character.visual_state.suit_variation)
        end
    end

    return taken_variations
end

local function get_possible_outfit_variations(outfit, taken_variations)
    local possible_variations = managers.blackmarket:get_all_suit_variations(outfit)
    if possible_variations == nil or next(possible_variations) == nil or #possible_variations == 1 then
        return { "default" }
    end

    local should_include_modded_outfit_variations = heist_outfit_selector:should_include_modded_outfit_variations()
    local should_use_unique_outfit_variations = heist_outfit_selector:should_use_unique_outfit_variations()
    for i = #possible_variations, 1, -1 do
        if (not should_include_modded_outfit_variations and is_modded_outfit_variation(outfit, possible_variations[i]))
                or (should_use_unique_outfit_variations and table.contains(taken_variations, possible_variations[i]))
                or is_outfit_or_variation_locked(outfit, possible_variations[i]) then
            table.remove(possible_variations, i)
        end
    end

    return possible_variations
end

local function apply_outfit_and_variation(visual_state, outfit)
    local function apply_style(style)
        visual_state.player_style = style
    end

    local function apply_variation(variation)
        visual_state.suit_variation = variation
    end

    if outfit == "heist_default" or is_outfit_or_variation_locked(outfit, "default") then
        return
    elseif outfit == "character_default" then
        apply_style("none")
        apply_variation("default")
        return
    else
        apply_style(outfit)
    end

    if heist_outfit_selector:should_use_default_outfit_variation() then
        apply_variation("default")
        return
    end

    local taken_variations = get_taken_outfit_variations(outfit)
    local possible_variations = get_possible_outfit_variations(outfit, taken_variations)

    if heist_outfit_selector:should_use_matching_outfit_variations() and next(taken_variations) ~= nil then
        apply_variation(taken_variations[1])
        return
    end

    if next(possible_variations) ~= nil then
        apply_variation(possible_variations[math.random(#possible_variations)])
    else
        apply_variation("default")
    end
end

local function apply_gloves(visual_state, outfit)
    local function apply_glove(glove)
        visual_state.glove_id = glove
    end

    local glove = heist_outfit_selector:get_glove_for_outfit(outfit)

    if glove == nil or glove == "outfit_default" or is_glove_locked(glove) then
        return
    elseif glove == "character_default" then
        apply_glove("default")
        return
    else
        apply_glove(glove)
    end
end

Hooks:PreHook(CriminalsManager,
    "update_character_visual_state",
    "HeistOutfitSelectorCriminalsManagerVisualState",
    function(self, character_name, visual_state)
        if visual_state == nil or next(visual_state) == nil then
            return
        end

        local current_level_id = managers.job:current_level_id()
        local outfit = heist_outfit_selector:get_outfit_for_level(current_level_id)
        if outfit == nil then
            return
        end

        local is_the_player = visual_state.is_local_peer == true
        local should_override_self = heist_outfit_selector:should_override_own_outfit()
        local should_override_crew = heist_outfit_selector:should_override_crew_outfits()

        local should_override = (is_the_player and should_override_self) or (not is_the_player and should_override_crew)

        local already_has_an_outfit_equipped = visual_state.player_style ~= nil and visual_state.player_style ~= "none"
        if not already_has_an_outfit_equipped or should_override then
            apply_outfit_and_variation(visual_state, outfit)
        end

        local already_has_gloves_equipped = visual_state.glove_id ~= nil and visual_state.glove_id ~= "default"
        if not already_has_gloves_equipped or should_override then
            apply_gloves(visual_state, outfit)
        end
    end)
